#!/usr/bin/python
#
# tardump.py
#
# (C)2002 Lars Gustbel (lars@gustaebel.de)
#
# $Id: tardump.py,v 1.4 2002/11/01 10:06:45 lars Exp $

import sys
import os
import time
import tarfile

if __name__ == "__main__":

    usage = """display detailed information on
POSIX 1003.1-1990 compliant tar files

Usage: %s [options] [files]

-h        display this help message
-z        tar archive is gzip compressed
-j        tar archive is bzip2 compressed
-f FILE   dump tar file `FILE`, else STDIN
    """ % sys.argv[0]

    import getopt
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hzjf:")
    except getopt.GetoptError, e:
        print
        print "ERROR:", e
        print usage
        sys.exit(0)

    fn = None
    comp = ""
    for o, a in opts:
        if o == "-f": fn = a
        if o == "-z": comp = "gz"
        if o == "-j": comp = "bz2"
        if o == "-h":
            print usage
            sys.exit(1)

    if fn is None:
        tar = tarfile.open(mode="r|"+comp, fileobj=sys.stdin)
        try:
            import msvcrt
            msvcrt.setmode(0, os.O_BINARY)
        except ImportError:
            pass
    else:
        try:
            tar = tarfile.open(fn, mode="r|"+comp)
        except tarfile.ReadError:
            print "File does not appear to be a tar archive"
            sys.exit(1)

    tar.debug = 3

    membercount = 0
    dirs = 0
    files = 0

    for t in tar:
        print "0x%06x" % t.offset, "-" * 50

        print "Name     :", repr(t.name)[1:-1]

        print "Type     :",
        if t.isreg():
            print "regular file"
            files += 1
        elif t.isdir():
            print "directory"
            dirs += 1
        elif t.issym():
            print "symbolic link"
        elif t.islnk():
            print "hard link"
        elif t.ischr():
            print "character device"
        elif t.isblk():
            print "block device"
        elif t.isfifo():
            print "fifo"

        print "Mode     :", oct(t.mode), "(%s)" % tarfile.filemode(t.mode)
        print "Mod.time :", time.asctime(time.localtime(t.mtime))
        print "Size     :",
        if t.size > 1024*1024:
            print t.size/1024/1024, "MB",
        if t.size > 1024:
            print t.size/1024, "KB",
        if t.size < 1024:
            print "%d bytes" % t.size,
        else:
            print "(%d bytes)" % t.size,

        bl = t.size/512
        if t.size % 512:
            bl += 1
        print "(%d blocks)" % bl


        if t.linkname:
            print "Linkname :", repr(t.linkname)[1:-1]

        print "uid, gid : %d/%d" % (t.uid, t.gid)
        print "Owner    : %s/%s" % (t.uname, t.gname)

        print "Checksum :", t.chksum

        if t.isdev():
            print "devmajor :", t.devmajor
            print "devminor :", t.devminor

        if t.prefix:
            print "Prefix   :", repr(t.prefix)[1:-1]

        if t.issparse():
            print "Sparse   : yes"

        membercount += 1
        print

    print
    print "Summary :",
    import gzip
    if isinstance(tar.fileobj, gzip.GzipFile):
        print "gzipped",
    print "tar archive"
    print "         ", membercount, "members in", dirs, "directories and", files, "files."
    print "         ", tar.offset/512, "blocks"

    tar.close()
